# vim:ts=4 sw=4 ft=tcl
#
# $Id: benchlib.tcl,v 1.2 2000/04/28 22:04:27 soul Exp $
#
# Copyright  2000, Laurent Duperval <lduperval@sprint.ca>
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#
#   * Redistributions in binary form must reproduce the above
#     copyright notice, this list of conditions and the following
#     disclaimer in the documentation and/or other materials provided
#     with the distribution.
#
#   * The name of the Copyright holder may not be used to endorse or promote
#     products derived from this software without specific prior written
#     permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

proc putsn {string} {
	puts -nonewline "$string"
}

# Set some platform specific stuff
puts "Version: [info patchlevel]\n--------"

# Not all interpreters support [file delete]
if {[info exists tcl_platform(platform)]} {
	if {$tcl_platform(platform) == "unix"} {
		set deleteCommand "/bin/rm"
	} elseif {$tcl_platform(platform) == "windows"} {
		set deleteCommand "$env(COMSPEC) /c del"
	}
} else {
	# The Good Ol' Days (?) when only Unix support existed
	set deleteCommand "/bin/rm"
}

# We need to look in the path to locate interpreters.
if {[info exists tcl_platform(platform)]} {
	if {$tcl_platform(platform) == "unix"} {
		set pathSep ":"
	} elseif {$tcl_platform(platform) == "windows"} {
		set pathSep ";"
	}
} else {
	# The Good Ol' Days (?) when only Unix support existed
	set pathSep :
}

# This creates a specific benchmark

proc bench {benchname desc body} {
	global benchdesc
	proc $benchname {} $body
	set benchdesc($benchname) $desc
}

proc benchargs {benchname argslist desc body} {
	global benchdesc
	proc $benchname $argslist $body
	set benchdesc($benchname) $desc
}

proc runbench {bench times} {
	if {[llength [split $bench]] > 1} {
		putsn "[lindex $bench 0]:\t"
		puts "[time {[lindex $bench 0] [lrange $bench 1 end]} $times]"
	} else {
		putsn "$bench:\t"
		puts "[time {$bench} $times]"
	}
}

proc comma {num {sep ,}} {
# - comma
# @c Separates a large number in groups of three digits or less.
#
# - Arguments
# @a num: the number to split
# @a sep: The separator to use
#
# - Returns
# @r The number with <a sep> inserted at the proper location.
#
   regexp {([-+0-9]*)([\.0-9]*)} $num {} int frac
   set len [string len $int]

   set n [expr ($len + 2) % 3]
   set r [string range $int 0 [expr $n]]

   for {set i [expr $n+1]} {$i < $len} {incr i 3} {
	   append r "$sep[string range $int $i [expr $i + 2]]"     }
   if {$frac > 0} {
	   append r $frac
   }

   set r
}
